import { mclAuthenticatedApiRequest } from "@mcl/api/requests";

// ===== API Requests =====

export const getServerFiles = (serverId: string, path: string) =>
  mclAuthenticatedApiRequest({
    subPath: "/files/list",
    json: { id: serverId, path },
  });

export const createServerFolder = (serverId: string, path: string) =>
  mclAuthenticatedApiRequest({
    subPath: "/files/folder",
    json: { id: serverId, path },
  });

export const uploadServerItem = (body:FormData) =>
  mclAuthenticatedApiRequest({
    subPath: "/files/upload",
    body,
  })

export const deleteServerItem = (
  serverId: string,
  path: string,
  isDir: boolean,
) =>
  mclAuthenticatedApiRequest({
    subPath: "/files/item",
    json: { id: serverId, path, isDir },
    method: "DELETE",
  });

export const moveServerItems = (
  serverId: string,
  files: string,
  destination: string,
  origin: string,
) =>
  mclAuthenticatedApiRequest({
    subPath: "/files/move",
    json: { id: serverId, files, destination, origin },
  });

export const previewServerItem = (serverId: string, path: string) =>
  mclAuthenticatedApiRequest({
    subPath: "/files/item",
    json: { id: serverId, path },
  }).then((res) => res.blob());

export const getServerItem = (serverId: string, name: string, path: string) =>
  mclAuthenticatedApiRequest({
    subPath: "/files/item",
    json: { id: serverId, path },
  })
    .then((res) => res.blob())
    .then((blob) => downloadBlob(blob, name));

async function downloadBlob(blob: Blob, name: string) {
  const url = window.URL.createObjectURL(blob);
  const a = document.createElement("a");
  a.style.display = "none";
  a.href = url;
  a.download = name;
  document.body.appendChild(a);
  a.click();
  window.URL.revokeObjectURL(url);
}
