import {
  useMutation,
  useQueryClient,
} from "@tanstack/react-query";
import { cairoAuthHeader } from "./auth";

declare interface ApiRequestArgs {
  subPath: string;
  body?: any | undefined;
  json?: object | undefined;
  method?: string;
  jsonify?: boolean;
  extraHeaders?: object;
  handleErrors?: boolean;
}

async function apiRequest(apiRequestArgs: ApiRequestArgs) {
  const { subPath, json, body, method, jsonify, extraHeaders, handleErrors } =
    apiRequestArgs;
  const headers = extraHeaders ?? {};
  const requestMethod = method ?? !!json ? "POST" : "GET";
  const requestBody = body ?? !!json ? JSON.stringify(json) : undefined;
  if (!!json) headers["Content-Type"] = "application/json";
  return fetch(`/api${subPath}`, {
    method: requestMethod,
    body: requestBody
  }).then((res) => {
    if (res.status >= 300 && handleErrors) throw Error(res.statusText);
    if (jsonify) return res.json();
    return res;
  });
}

export async function mclAuthenticatedApiRequest(
  apiRequestArgs: ApiRequestArgs,
) {
  const extraHeaders = apiRequestArgs.extraHeaders ?? {};
  const authHeaders = cairoAuthHeader();
  apiRequestArgs.extraHeaders = { ...extraHeaders, ...authHeaders };
  return apiRequest(apiRequestArgs);
}

export async function mclApiRequest(apiRequestArgs: ApiRequestArgs) {
  return apiRequest(apiRequestArgs);
}

export const useMutator = async (
  apiRequest: (args: any) => Promise<any>,
  invalidate: string[],
) => {
  const qc = useQueryClient();
  useMutation({
    mutationFn: apiRequest,
    onSuccess: () => qc.invalidateQueries({ queryKey: invalidate }),
  });
};
